@@ -49,11 +49,38 @@ package Sidef::Deparse::Perl {
             #~ to_num  => q{0+},
             #~ },
 
-            special_constructs => {
-                                   'Sidef::Types::Block::If'    => 1,
-                                   'Sidef::Types::Block::While' => 1,
-                                   'Sidef::Types::Block::For'   => 1,
-                                  },
+            overload_methods => {
+                                 '=='  => 'eq',
+                                 '<=>' => 'cmp',
+                                 '~~'  => '~~',
+                                },
+
+            data_types => {
+                qw(
+                  Sidef::DataTypes::Bool::Bool            Sidef::Types::Bool::Bool
+                  Sidef::DataTypes::Array::Array          Sidef::Types::Array::Array
+                  Sidef::DataTypes::Array::Pair           Sidef::Types::Array::Pair
+                  Sidef::DataTypes::Array::MultiArray     Sidef::Types::Array::MultiArray
+                  Sidef::DataTypes::Hash::Hash            Sidef::Types::Hash::Hash
+                  Sidef::DataTypes::Regex::Regex          Sidef::Types::Regex::Regex
+                  Sidef::DataTypes::String::String        Sidef::Types::String::String
+                  Sidef::DataTypes::Number::Number        Sidef::Types::Number::Number
+                  Sidef::DataTypes::Number::Complex       Sidef::Types::Number::Complex
+                  Sidef::DataTypes::Range::RangeNumber    Sidef::Types::Range::RangeNumber
+                  Sidef::DataTypes::Range::RangeString    Sidef::Types::Range::RangeString
+                  Sidef::DataTypes::Block::Block          Sidef::Types::Block::Block
+                  Sidef::DataTypes::Glob::Socket          Sidef::Types::Glob::Socket
+                  Sidef::DataTypes::Glob::Pipe            Sidef::Types::Glob::Pipe
+                  Sidef::DataTypes::Glob::Backtick        Sidef::Types::Glob::Backtick
+                  Sidef::DataTypes::Glob::DirHandle       Sidef::Types::Glob::DirHandle
+                  Sidef::DataTypes::Glob::FileHandle      Sidef::Types::Glob::FileHandle
+                  Sidef::DataTypes::Glob::Dir             Sidef::Types::Glob::Dir
+                  Sidef::DataTypes::Glob::File            Sidef::Types::Glob::File
+                  Sidef::DataTypes::Object::Object        Sidef::Object::Object
+                  Sidef::DataTypes::Sidef::Sidef          Sidef
+                  Sidef::DataTypes::Variable::LazyMethod  Sidef::Variable::LazyMethod
+                  )
+            },
 
             reassign_ops => {map (("$_=" => $_), qw(+ - % * // / & | ^ ** && || << >> ÷))},
 
@@ -73,7 +100,7 @@ HEADER
 
         if (exists $opts{opt}{P}) {
             my $precision = abs(int($opts{opt}{P}));
-            $opts{header} .= "local \$Sidef::Types::Number::Number::PREC = 4*$precision;\n";
+            $opts{header} .= "BEGIN { \$Sidef::Types::Number::Number::PREC = 4*$precision; }\n";
         }
 
         if (exists $opts{opt}{M}) {
@@ -140,7 +167,21 @@ HEADER
         my ($self, $obj) = @_;
 
         my $ref = ref($obj);
+
+        if (exists $self->{data_types}{$ref}) {
+            my $target = $self->{data_types}{$ref};
+
+            state $cache = {};
+            $cache->{$ref} //= do {
+                $self->{header} .= "defined(&$target\::new) || require $target;\n";
+                1;
+            };
+
+            return $target;
+        }
+
             $ref eq 'Sidef::Variable::ClassInit'     ? $self->_dump_class_name($obj)
+          : $ref eq 'Sidef::Variable::Struct'        ? $self->_dump_class_name($obj)
           : $ref eq 'Sidef::Variable::Ref'           ? 'REF'
           : $ref eq 'Sidef::Types::Block::BlockInit' ? 'Sidef::Types::Block::Block'
           :                                            $ref;
@@ -168,6 +209,15 @@ HEADER
         $d;
     }
 
+    sub _dump_op_call {
+        my ($self, $method) = @_;
+
+        my $name = 'OP' . join('', unpack('C*', $method)) . refaddr($self);
+        my $line = "use constant $name => \\" . $self->_dump_string($method) . ";\n";
+        $self->top_add($line);
+        '->${(' . $self->{environment_name} . '::' . $name . ')}';
+    }
+
     sub _dump_var {
         my ($self, $var, $refaddr) = @_;
 
@@ -448,7 +498,7 @@ HEADER
 
     sub _dump_class_name {
         my ($self, $class) = @_;
-        join('::', '_', $class->{class}, $class->{name});
+        join('::', $self->{environment_name}, $class->{class}, $class->{name});
     }
 
     sub deparse_generic {
@@ -469,18 +519,13 @@ HEADER
         $self->deparse_generic('(', ', ', ')', @args);
     }
 
-    sub deparse_block_expr {
-        my ($self, @args) = @_;
-        $self->deparse_generic('do{', ';', '}', @args);
-    }
-
     sub deparse_bare_block {
         my ($self, @args) = @_;
 
         $Sidef::SPACES += $Sidef::SPACES_INCR;
-        my $code = $self->deparse_generic("{\n" . " " x ($Sidef::SPACES),
-                                          ";\n" . (" " x ($Sidef::SPACES)),
-                                          "\n" .  (" " x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "}", @args);
+        my $code = $self->deparse_generic("{\n" . (" " x $Sidef::SPACES),
+                                          ";\n" . (" " x $Sidef::SPACES),
+                                          "\n" . (" " x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "}", @args);
         $Sidef::SPACES -= $Sidef::SPACES_INCR;
 
         $code;
@@ -526,7 +571,7 @@ HEADER
 
                     # Anonymous function
                     if ($obj->{name} eq '') {
-                        $obj->{name} = "__ANON__";
+                        $obj->{name} = "__FUNC__";
                     }
 
                     my $name = $obj->{name};
@@ -590,7 +635,7 @@ HEADER
                         }
 
                         # Anonymous method
-                        elsif ($obj->{name} eq '__ANON__') {
+                        elsif ($obj->{name} eq '__FUNC__') {
                             ## don't add anonymous methods to the class,
                             ## but allow them to be defined and used freely
                         }
@@ -605,12 +650,18 @@ HEADER
                         }
 
                         # Add the "overload" pragma for some special methods
-                        #~ if (exists $self->{overload_methods}{$obj->{name}}) {
-                        #~ $code .= ";\n"
-                        #~ . (' ' x $Sidef::SPACES)
-                        #~ . qq{use overload q{$self->{overload_methods}{$obj->{name}}} => }
-                        #~ . $self->_dump_string("$self->{package_name}::$obj->{name}");
-                        #~ }
+                        if (exists $self->{overload_methods}{$name}) {
+                            my $overload_name = $self->{overload_methods}{$name};
+                            $code .= ";\n"
+                              . (' ' x $Sidef::SPACES)
+                              . qq(use overload q{$overload_name} =>)
+                              . q( sub { my ($first, $second, $swap) = @_; )
+                              . q( if ($swap) { ($first, $second) = ($second, $first) } )
+                              . q( $first)
+                              . $self->_dump_op_call($name)
+                              . q(($second) }; );
+                        }
+
                     }
                 }
             }
@@ -693,12 +744,12 @@ HEADER
                         die "[ERROR] Invalid class name: '$obj->{name}' inside namespace '$obj->{class}'";
                     }
 
-                    $code .= ($package_name = ref($obj->{name}));
+                    $code .= ($package_name = $self->_get_reftype($obj->{name}));
                 }
                 else {
 
                     if ($obj->{name} eq '') {
-                        $obj->{name} = '__ANON__' . $refaddr;
+                        $obj->{name} = '__FUNC__' . $refaddr;
                     }
 
                     $code .= ($package_name = $self->_dump_class_name($obj));
@@ -946,7 +997,7 @@ HEADER
             $code = $self->_dump_array('Sidef::Types::Array::Array', $obj);
         }
         elsif ($ref eq 'Sidef::Types::Bool::Bool') {
-            $code = $self->make_constant($ref, 'new', ${$obj} ? ("true$refaddr", 1) : ("false$refaddr", 0));
+            $code = 'Sidef::Types::Bool::Bool::' . (${$obj} ? 'TRUE' : 'FALSE');
         }
         elsif ($ref eq 'Sidef::Types::Regex::Regex') {
             $code =
@@ -954,14 +1005,107 @@ HEADER
                                    $self->_dump_string("$obj->{raw}"),
                                    $self->_dump_string($obj->{flags} . ($obj->{global} ? 'g' : '')));
         }
-        elsif ($ref eq 'Sidef::Types::Block::If' or $ref eq 'Sidef::Types::Block::While') {
-            ## ok
+        elsif ($ref eq 'Sidef::Types::Block::If') {
+            $code = "do{";
+            foreach my $i (0 .. $#{$obj->{if}}) {
+                $code .= ($i == 0 ? 'if' : 'elsif');
+                my $info = $obj->{if}[$i];
+                $code .= "(do{" . $self->deparse_args($info->{expr}) . "})" . $self->deparse_bare_block($info->{block}{code});
+            }
+            if (exists $obj->{else}) {
+                $code .= 'else' . $self->deparse_bare_block($obj->{else}{block}{code});
+            }
+            $code .= '}';
+        }
+        elsif ($ref eq 'Sidef::Types::Block::While') {
+            $code = "while(do{" . $self->deparse_args($obj->{expr}) . "})" . $self->deparse_bare_block($obj->{block}{code});
+        }
+        elsif ($ref eq 'Sidef::Types::Block::ForEach') {
+            $code = $self->deparse_args($obj->{expr}) . '->each' . '(' . $self->deparse_expr({self => $obj->{block}}) . ')';
+        }
+        elsif ($ref eq 'Sidef::Types::Block::CFor') {
+            $code = 'for('
+              . join(';', map { $self->deparse_args($_) } @{$obj->{expr}}) . ')'
+              . $self->deparse_bare_block($obj->{block}{code});
+        }
+        elsif ($ref eq 'Sidef::Types::Block::ForIn') {
+
+            my $expr = $self->deparse_expr({self => $obj->{expr}});
+
+            if (
+                @{$obj->{vars}} > 1
+                or (@{$obj->{vars}} == 1
+                    and exists($obj->{vars}[0]{slurpy}))
+              ) {
+
+                my $last_var = $obj->{vars}[-1];
+
+                my $hash  = 0;
+                my $array = 0;
+                if (exists($last_var->{slurpy})) {
+                    delete($last_var->{slurpy});
+                    if (exists($last_var->{array})) {
+                        $array = delete($last_var->{array});
+                    }
+                    else {
+                        $hash = delete($last_var->{hash});
+                    }
+                }
+
+                my @vars = map { $self->deparse_expr({self => $_}) } @{$obj->{vars}};
+
+                my @last_var = ($array ? ($vars[-1] =~ tr/$/@/r) : $hash ? ($vars[-1] =~ tr/$/%/r) : ());
+                my $init_vars = '(' . join(',', @vars, @last_var) . ')';
+                my $vars = '(' . join(',', @vars[0 .. $#vars - @last_var], @last_var) . ')';
+                my $block = $self->deparse_bare_block($obj->{block}->{code});
+
+                $code =
+                    'do { my $obj = '
+                  . $expr . '; my '
+                  . $init_vars . '; '
+                  . 'my $block = CORE::sub '
+                  . $block . ';'
+                  . 'for my $group(ref($obj) ? $obj->SUPER::isa("ARRAY") ? @$obj : @{$obj->to_a} : ()) {'
+                  . $vars
+                  . ' = (ref($group) && $group->SUPER::isa("ARRAY") ? @$group'
+                  . ' : $group);'
+                  . (
+                     @last_var
+                     ? ($vars[-1] . ' = '
+                        . ($array ? 'Sidef::Types::Array::Array' : 'Sidef::Types::Hash::Hash')
+                        . '->new('
+                        . join('', @last_var) . ')')
+                     : ''
+                    )
+                  . '; $block->() }}';
+            }
+            else {
+
+                my @vars = map { $self->deparse_expr({self => $_}) } @{$obj->{vars}};
+                my $block = $self->deparse_bare_block($obj->{block}->{code});
+
+                my $var = $vars[0];
+
+                $code =
+                    'do { my $obj = '
+                  . $expr . '; my '
+                  . $var
+                  . '; my $block = CORE::sub '
+                  . $block . ';'
+                  . 'if (ref($obj)) { if (defined(my $sub = $obj->SUPER::can("range_iter"))) { my $iter = $sub->($obj);'
+                  . 'while (defined('
+                  . $var
+                  . '  = $iter->())) { $block->()} }'
+                  . 'else { for my $item ($obj->SUPER::isa("ARRAY") ? @$obj : @{$obj->to_a}) { '
+                  . $var
+                  . ' = $item; $block->() }}}}';
+            }
         }
         elsif ($ref eq 'Sidef::Types::Bool::Ternary') {
             $code = '('
-              . $self->deparse_script($obj->{cond}) . '?'
-              . $self->deparse_block_expr($obj->{true}) . ':'
-              . $self->deparse_block_expr($obj->{false}) . ')';
+              . $self->deparse_script($obj->{cond}) . '?' . 'do'
+              . $self->deparse_bare_block($obj->{true}) . ':' . 'do'
+              . $self->deparse_bare_block($obj->{false}) . ')';
         }
         elsif ($ref eq 'Sidef::Variable::NamedParam') {
             $code = $ref . '->new(' . $self->_dump_string($obj->[0]) . ', ' . $self->deparse_args(@{$obj->[1]}) . ')';
@@ -1033,9 +1177,6 @@ HEADER
         elsif ($ref eq 'Sidef::Variable::Ref') {
             ## ok
         }
-        elsif ($ref eq 'Sidef::Types::Block::For') {
-            ## ok
-        }
         elsif ($ref eq 'Sidef::Types::Block::Break') {
             $code = 'last';
         }
@@ -1118,15 +1259,8 @@ HEADER
                                          "'" . $obj->im->get_value . "'");
         }
         elsif ($ref eq 'Sidef::Types::Array::Pair') {
-            if (all { not defined($_) } @{$obj}) {
-                $code = $self->make_constant($ref, 'new', "Pair$refaddr");
-            }
-            else {
-                $code =
-                    $ref
-                  . '->new('
-                  . join(', ', map { defined($_) ? $self->deparse_expr({self => $_}) : 'undef' } @{$obj}) . ')';
-            }
+            $code =
+              $ref . '->new(' . join(', ', map { defined($_) ? $self->deparse_expr({self => $_}) : 'undef' } @{$obj}) . ')';
         }
         elsif ($ref eq 'Sidef::Types::Null::Null') {
             $code = $self->make_constant($ref, 'new', "Null$refaddr");
@@ -1137,8 +1271,21 @@ HEADER
         elsif ($ref eq 'Sidef::Module::Func') {
             $code = $self->make_constant($ref, '__NEW__', "MOD_F$refaddr", $self->_dump_string($obj->{module}));
         }
-        elsif ($ref eq 'Sidef::Types::Range::RangeNumber' or $ref eq 'Sidef::Types::Range::RangeString') {
-            $code = $ref . '->new';
+        elsif ($ref eq 'Sidef::Types::Range::RangeNumber') {
+            $code = $self->make_constant(
+                $ref,
+                '__new__',
+                "RangeNumber$refaddr",
+                map {
+                        "$_ => do { my \$r = Math::GMPq::Rmpq_init(); Math::GMPq::Rmpq_set_str(\$r,'"
+                      . Math::GMPq::Rmpq_get_str($obj->{$_}, 10)
+                      . "', 10); \$r }"
+                  } keys(%{$obj})
+            );
+        }
+        elsif ($ref eq 'Sidef::Types::Range::RangeString') {
+            $code = $self->make_constant($ref, '__new__', "RangeString$refaddr",
+                                         map { "$_ => " . $self->_dump_string($obj->{$_}) } keys(%{$obj}));
         }
         elsif ($ref eq 'Sidef::Types::Glob::Backtick') {
             $code = $self->make_constant($ref, 'new', "Backtick$refaddr", $self->_dump_string(${$obj}));
@@ -1190,19 +1337,6 @@ HEADER
             $code = qq~((CORE::warn(@args, " at \Q$obj->{file}\E line $obj->{line}\\n")) ? ~
               . qq~(Sidef::Types::Bool::Bool::FALSE) : (Sidef::Types::Bool::Bool::TRUE))~;
         }
-        elsif ($ref eq 'Sidef::Object::Object') {
-            $code = $self->make_constant($ref, 'new', "Object$refaddr");
-        }
-        elsif ($ref eq 'Sidef::Variable::LazyMethod') {
-            $code = $ref . '->new';
-        }
-        elsif ($ref eq 'Sidef::Types::Block::ForArray') {
-            $code =
-                'for my '
-              . $self->deparse_expr({self => $obj->{var}}) . '(@{'
-              . $self->deparse_expr({self => $obj->{array}}) . '})'
-              . $self->deparse_bare_block($obj->{block}->{code});
-        }
         elsif ($ref eq 'Sidef::Types::Array::MultiArray') {
             $code = $self->make_constant($ref, 'new', "MultiArr$refaddr");
         }
@@ -1221,6 +1355,9 @@ HEADER
         elsif ($ref eq 'Sidef::Meta::Unimplemented') {
             $code = qq{CORE::die "Unimplemented at " . } . $self->_dump_string($obj->{file}) . qq{. " line $obj->{line}\\n"};
         }
+        elsif (exists $self->{data_types}{$ref}) {
+            $code = "'" . $self->{data_types}{$ref} . "'";
+        }
 
         # Array and hash indices
         if (exists $expr->{ind}) {
@@ -1272,9 +1409,7 @@ HEADER
                 my $method = $call->{method};
 
                 if ($code ne '') {
-                    if (not exists $self->{special_constructs}{$ref}) {
-                        $code = '(' . $code . ')';
-                    }
+                    $code = '(' . $code . ')';
                 }
 
                 if ($ref eq 'Sidef::Types::Block::Return') {
@@ -1339,7 +1474,7 @@ HEADER
                     }
 
                     if (exists($self->{lazy_ops}{$method})) {
-                        $code .= $self->{lazy_ops}{$method} . $self->deparse_block_expr(@{$call->{arg}});
+                        $code .= $self->{lazy_ops}{$method} . 'do' . $self->deparse_bare_block(@{$call->{arg}});
                         next;
                     }
 
@@ -1353,7 +1488,8 @@ HEADER
                     if (exists $self->{reassign_ops}{$method}) {
 
                         $code =
-                          "CORE::sub : lvalue {my \$ref=\\$code; \$\$ref=\$\$ref\->\${\\'$self->{reassign_ops}{$method}'}"
+                            "CORE::sub : lvalue {my \$ref=\\$code; \$\$ref=\$\$ref"
+                          . $self->_dump_op_call($self->{reassign_ops}{$method})
                           . $self->deparse_args(@{$call->{arg}}) . "}->()";
 
                         #$code =
@@ -1370,28 +1506,12 @@ HEADER
                     # != and == methods
                     if ($method eq '==' or $method eq '!=') {
                         $code =
-                            '(' . 'do{'
+                            'do{ my $bool = do{'
                           . $code
                           . '} eq do{'
-                          . $self->deparse_args(@{$call->{arg}}) . '} ? '
-                          . (
-                             $method eq '!='
-                             ? '(Sidef::Types::Bool::Bool::FALSE) : (Sidef::Types::Bool::Bool::TRUE)'
-                             : '(Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE)'
-                            )
-                          . ')';
-                        next;
-                    }
-
-                    # <=> method
-                    if ($method eq '<=>') {
-                        $code =
-                            '((Sidef::Types::Number::Number::ZERO,'
-                          . 'Sidef::Types::Number::Number::ONE,'
-                          . 'Sidef::Types::Number::Number::MONE)[do{'
-                          . $code
-                          . '} cmp do {'
-                          . $self->deparse_args(@{$call->{arg}}) . '}])';
+                          . $self->deparse_args(@{$call->{arg}})
+                          . '}; ref($bool) ? $bool : ($bool ? Sidef::Types::Bool::Bool::TRUE : Sidef::Types::Bool::Bool::FALSE) }'
+                          . ($method eq '!=' ? '->neg' : '');
                         next;
                     }
 
@@ -1399,16 +1519,24 @@ HEADER
                     if ($method eq '~~' or $method eq '!~') {
                         $self->top_add(qq{use experimental 'smartmatch';\n});
                         $code =
-                            '(' . 'do{'
+                            'do{ my $bool = do{'
                           . $code
                           . '} ~~ do{'
-                          . $self->deparse_args(@{$call->{arg}}) . '} ? '
-                          . (
-                             $method eq '!~'
-                             ? '(Sidef::Types::Bool::Bool::FALSE) : (Sidef::Types::Bool::Bool::TRUE)'
-                             : '(Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE)'
-                            )
-                          . ')';
+                          . $self->deparse_args(@{$call->{arg}})
+                          . '}; ref($bool) ? $bool : ($bool ? Sidef::Types::Bool::Bool::TRUE : Sidef::Types::Bool::Bool::FALSE) }'
+                          . ($method eq '!~' ? '->neg' : '');
+                        next;
+                    }
+
+                    # <=> method
+                    if ($method eq '<=>') {
+                        $code =
+                            'do { my $cmp = do { '
+                          . $code
+                          . '} cmp do {'
+                          . $self->deparse_args(@{$call->{arg}})
+                          . '}; ref($cmp) ? $cmp : ($cmp < 0 ? Sidef::Types::Number::Number::MONE : '
+                          . '$cmp > 0 ? Sidef::Types::Number::Number::ONE : Sidef::Types::Number::Number::ZERO )}';
                         next;
                     }
 
@@ -1441,6 +1569,11 @@ HEADER
                             next;
                         }
 
+                        if ($method eq '^') {
+                            $code = $self->deparse_args(@{$call->{arg}}) . '->range';
+                            next;
+                        }
+
                         if ($method eq 'say' or $method eq '>') {
                             $code =
                                 '((CORE::say'
@@ -1456,6 +1589,14 @@ HEADER
                               . ') ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE))';
                             next;
                         }
+
+                        if ($method eq 'defined') {
+                            $code =
+                                '((CORE::defined'
+                              . $self->deparse_args(@{$call->{arg}})
+                              . ') ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE))';
+                            next;
+                        }
                     }
 
                     if (ref($method)) {
@@ -1500,7 +1641,12 @@ HEADER
 
                         # Operator-like method call
                         else {
-                            $code .= '->${\\' . q{'} . $method . q{'} . '}';
+
+                            ## Old way:
+                            #$code .= '->${\\' . q{'} . $method . q{'} . '}';
+
+                            ## New way:
+                            $code .= $self->_dump_op_call($method);
                         }
                     }
                 }
@@ -1510,23 +1656,11 @@ HEADER
                 }
 
                 if (exists $call->{arg}) {
-                    if ($ref eq 'Sidef::Types::Block::For') {
-                        $code .= $self->deparse_generic('(', ';', ')', @{$call->{arg}});
-                    }
-                    else {
-                        $code .= $self->deparse_args(@{$call->{arg}});
-                    }
+                    $code .= $self->deparse_args(@{$call->{arg}});
                 }
 
                 if (exists $call->{block}) {
-
-                    # TODO: move above the deparsing of `Block::If`
-                    if ($ref eq 'Sidef::Types::Block::If' and $i == $end) {
-                        $code = "do {\n" . (' ' x $Sidef::SPACES) . $code . $self->deparse_bare_block(@{$call->{block}}) . '}';
-                    }
-                    else {
-                        $code .= $self->deparse_bare_block(@{$call->{block}});
-                    }
+                    $code .= $self->deparse_bare_block(@{$call->{block}});
                     next;
                 }
             }
