@@ -2,7 +2,9 @@ package Sidef::Object::Object {
 
     use 5.014;
     require Scalar::Util;
+
     use Sidef::Types::Bool::Bool;
+    use Sidef::Types::Number::Number;
 
     use overload
       q{~~}   => \&{__PACKAGE__ . '::' . '~~'},
@@ -40,25 +42,30 @@ package Sidef::Object::Object {
         if (CORE::ref($obj1) && $obj1->SUPER::isa(CORE::ref($obj2)) or CORE::ref($obj2) && $obj2->SUPER::isa(CORE::ref($obj1)))
         {
             if (defined(my $sub = $obj1->can('<=>'))) {
-                my $result = $sub->($obj1, $obj2);
-                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                return $result->get_value;
+                return $sub->($obj1, $obj2);
             }
         }
 
-        Scalar::Util::refaddr($obj1) <=> (CORE::ref($obj2) ? Scalar::Util::refaddr($obj2) : 'inf');
+        my $cmp = Scalar::Util::refaddr($obj1) <=> (CORE::ref($obj2) ? Scalar::Util::refaddr($obj2) : 'inf');
+
+            $cmp < 0 ? Sidef::Types::Number::Number::MONE
+          : $cmp > 0 ? Sidef::Types::Number::Number::ONE
+          :            Sidef::Types::Number::Number::ZERO;
       },
       q{eq} => sub {
         my ($obj1, $obj2) = @_;
 
-        ($obj1->SUPER::isa(CORE::ref($obj2) || return) || $obj2->SUPER::isa(CORE::ref($obj1) || return))
-          || return;
+        (   $obj1->SUPER::isa(CORE::ref($obj2) || return (Sidef::Types::Bool::Bool::FALSE))
+         || $obj2->SUPER::isa(CORE::ref($obj1) || return (Sidef::Types::Bool::Bool::FALSE)))
+          || return (Sidef::Types::Bool::Bool::FALSE);
 
         if (defined(my $sub = $obj1->can('=='))) {
-            return ${$sub->($obj1, $obj2)};
+            return $sub->($obj1, $obj2);
         }
 
-        Scalar::Util::refaddr($obj1) == Scalar::Util::refaddr($obj2);
+        Scalar::Util::refaddr($obj1) == Scalar::Util::refaddr($obj2)
+          ? (Sidef::Types::Bool::Bool::TRUE)
+          : (Sidef::Types::Bool::Bool::FALSE);
       };
 
     sub new {
@@ -84,11 +91,6 @@ package Sidef::Object::Object {
         Sidef::Variable::LazyMethod->new(obj => $self, method => $method, args => \@args);
     }
 
-    #~ sub lazy {
-    #~ my ($self) = @_;
-    #~ Sidef::Lazy::Lazy->new($self);
-    #~ }
-
     sub object_id {
         my ($self) = @_;
         Sidef::Types::Number::Number->new(Scalar::Util::refaddr($self));
@@ -154,7 +156,7 @@ package Sidef::Object::Object {
         Sidef::Types::Array::Array->new(map { Sidef::Types::String::String->new($_) } $extract_parents->(CORE::ref($obj)));
     }
 
-    sub super_join {
+    sub interpolate {
         my ($self, @args) = @_;
         $self->new(
             CORE::join(
@@ -392,7 +394,7 @@ package Sidef::Object::Object {
         # Negation of smart match
         *{__PACKAGE__ . '::' . '!~'} = sub {
             state $method = '~~';
-            $_[0]->$method($_[1])->not;
+            $_[0]->$method($_[1])->neg;
         };
     }
 }
