@@ -45,6 +45,7 @@ package Sidef::Optimizer {
                  { $_, [table(STRING)] }
                } methods(STRING, qw(
                    new call
+                   to downto
 
                    concat
                    prepend
@@ -58,11 +59,13 @@ package Sidef::Optimizer {
                    crypt
 
                    levenshtein
+                   jaro_distance
                    contains
                    overlaps
                    begins_with
                    ends_with
                    count
+                   range
 
                    sprintf
                    sprintlnf
@@ -191,6 +194,16 @@ package Sidef::Optimizer {
                )
             ),
 
+            # String.method(String, Bool)
+            (
+             map {
+                 { $_, [table(STRING), table(BOOL)] }
+               } methods(STRING, qw(
+                   range
+                   )
+               )
+            ),
+
         ],
 
         (NUMBER) => [
@@ -230,6 +243,9 @@ package Sidef::Optimizer {
                    idiv
                    ipow
                    imod
+                   range
+                   to
+                   downto
 
                    shift_right
                    shift_left
@@ -352,8 +368,7 @@ package Sidef::Optimizer {
                  { $_, [table(NUMBER), table(NUMBER)] }
                } methods(NUMBER, qw(
                    modpow
-                   shift_right
-                   shift_left
+                   range
                    )
                )
             ),
@@ -675,7 +690,6 @@ package Sidef::Optimizer {
         }
         elsif (   $ref eq 'Sidef::Variable::ClassInit'
                or $ref eq 'Sidef::Types::Block::Do'
-               or $ref eq 'Sidef::Types::Block::ForArray'
                or $ref eq 'Sidef::Types::Block::Loop'
                or $ref eq 'Sidef::Types::Block::Given'
                or $ref eq 'Sidef::Types::Block::When'
@@ -710,9 +724,41 @@ package Sidef::Optimizer {
 
             # Has no expressions, so let's convert it into an Array
             if (not $has_expr) {
-                $obj = Sidef::Types::Array::Array->new(@{$obj});
+
+                #$obj = Sidef::Types::Array::Array->new(@{$obj});
+                bless $obj, 'Sidef::Types::Array::Array';
             }
         }
+        elsif ($ref eq 'Sidef::Types::Block::If') {
+            foreach my $i (0 .. $#{$obj->{if}}) {
+                my %code = $self->optimize($obj->{if}[$i]{block}{code});
+                $obj->{if}[$i]{block}{code} = \%code;
+
+                my %expr = $self->optimize($obj->{if}[$i]{expr});
+                $obj->{if}[$i]{expr} = \%expr;
+            }
+            if (exists $obj->{else}) {
+                my %code = $self->optimize($obj->{else}{block}{code});
+                $obj->{else}{block}{code} = \%code;
+            }
+        }
+        elsif (   $ref eq 'Sidef::Types::Block::While'
+               or $ref eq 'Sidef::Types::Block::ForIn'
+               or $ref eq 'Sidef::Types::Block::ForEach') {
+            my %expr = $self->optimize($obj->{expr});
+            $obj->{expr} = \%expr;
+
+            my %code = $self->optimize($obj->{block}{code});
+            $obj->{block}{code} = \%code;
+        }
+        elsif ($ref eq 'Sidef::Types::Block::CFor') {
+            foreach my $i (0 .. $#{$obj->{expr}}) {
+                my %expr = $self->optimize($obj->{expr}[$i]);
+                $obj->{expr}[$i] = \%expr;
+            }
+            my %code = $self->optimize($obj->{block}{code});
+            $obj->{block}{code} = \%code;
+        }
 
         if (not exists($expr->{ind}) and not exists($expr->{call})) {
             return (ref($obj) eq 'HASH' ? {self => $obj} : $obj);
