@@ -7,11 +7,6 @@ package Sidef::Types::Number::Number {
     use Math::GMPz qw();
     use Math::MPFR qw();
 
-    use parent qw(
-      Sidef::Object::Object
-      Sidef::Convert::Convert
-      );
-
     our $ROUND = Math::MPFR::MPFR_RNDN();
     our $PREC  = 128;
 
@@ -23,6 +18,11 @@ package Sidef::Types::Number::Number {
                   MONE => bless(\Math::GMPq->new(-1), __PACKAGE__),
                  };
 
+    use parent qw(
+      Sidef::Object::Object
+      Sidef::Convert::Convert
+      );
+
     use overload
       q{bool} => sub { Math::GMPq::Rmpq_sgn(${$_[0]}) != 0 },
       q{0+}   => sub { Math::GMPq::Rmpq_get_d(${$_[0]}) },
@@ -484,13 +484,15 @@ package Sidef::Types::Number::Number {
         my ($x, $y) = @_;
         _valid($y);
 
-        if (!Math::GMPq::Rmpq_sgn($$y)) {
-            my $sign = Math::GMPq::Rmpq_sgn($$x);
+        my $r = _big2mpz($x);
+        $y = _big2mpz($y);
+
+        if (!Math::GMPz::Rmpz_sgn($y)) {
+            my $sign = Math::GMPz::Rmpz_sgn($r);
             return (!$sign ? nan() : $sign > 0 ? inf() : ninf());
         }
 
-        my $r = _big2mpz($x);
-        Math::GMPz::Rmpz_div($r, $r, _big2mpz($y));
+        Math::GMPz::Rmpz_div($r, $r, $y);
         _mpz2big($r);
     }
 
@@ -2432,7 +2434,6 @@ package Sidef::Types::Number::Number {
         *{__PACKAGE__ . '::' . '!='}  = \&ne;
         *{__PACKAGE__ . '::' . '≠'} = \&ne;
         *{__PACKAGE__ . '::' . '..'}  = \&array_to;
-        *{__PACKAGE__ . '::' . '...'} = \&to;
         *{__PACKAGE__ . '::' . '..^'} = \&to;
         *{__PACKAGE__ . '::' . '^..'} = \&downto;
         *{__PACKAGE__ . '::' . '!'}   = \&factorial;
