@@ -59,22 +59,31 @@ package Sidef::Types::Range::RangeNumber {
     sub by {
         my ($self, $step) = @_;
         Sidef::Types::Number::Number::_valid($step);
-        $self->{step} = Math::GMPq::Rmpq_sgn($self->{step}) < 0 ? -$$step : $$step;
-        $self;
+        __PACKAGE__->__new__(
+                             from => $self->{from},
+                             to   => $self->{to},
+                             step => (Math::GMPq::Rmpq_sgn($self->{step}) < 0 ? -$$step : $$step),
+                            );
     }
 
     sub from {
         my ($self, $from) = @_;
         Sidef::Types::Number::Number::_valid($from);
-        $self->{from} = $$from;
-        $self;
+        __PACKAGE__->__new__(
+                             from => $$from,
+                             to   => $self->{to},
+                             step => $self->{step},
+                            );
     }
 
     sub to {
         my ($self, $to) = @_;
         Sidef::Types::Number::Number::_valid($to);
-        $self->{to} = $$to;
-        $self;
+        __PACKAGE__->__new__(
+                             from => $self->{from},
+                             to   => $$to,
+                             step => $self->{step},
+                            );
     }
 
     sub reverse {
@@ -84,10 +93,11 @@ package Sidef::Types::Range::RangeNumber {
             die "[ERROR] Can't reverse an infinite range: $self";
         }
 
-        $self->{step} = -$self->{step};
-        ($self->{from}, $self->{to}) = ($self->{to}, $self->{from});
-
-        $self;
+        __PACKAGE__->__new__(
+                             from => $self->{to},
+                             to   => $self->{from},
+                             step => -$self->{step},
+                            );
     }
 
     sub min {
@@ -166,7 +176,7 @@ package Sidef::Types::Range::RangeNumber {
     *include  = \&contains;
     *includes = \&contains;
 
-    sub _new_iter {
+    sub range_iter {
         my ($self) = @_;
 
         my $step = $self->{step};
@@ -198,7 +208,7 @@ package Sidef::Types::Range::RangeNumber {
     sub each {
         my ($self, $code) = @_;
 
-        my $iter = $self->_new_iter();
+        my $iter = $self->range_iter();
         while (defined(my $num = $iter->())) {
             if (defined(my $res = $code->_run_code($num))) {
                 return $res;
@@ -213,7 +223,7 @@ package Sidef::Types::Range::RangeNumber {
         my ($self, $code) = @_;
 
         my $values = Sidef::Types::Array::Array->new;
-        my $iter   = $self->_new_iter();
+        my $iter   = $self->range_iter();
         while (defined(my $num = $iter->())) {
             push @$values, $code->run($num);
         }
@@ -227,7 +237,7 @@ package Sidef::Types::Range::RangeNumber {
         my ($self, $code) = @_;
 
         my $values = Sidef::Types::Array::Array->new;
-        my $iter   = $self->_new_iter();
+        my $iter   = $self->range_iter();
         while (defined(my $num = $iter->())) {
             push(@$values, $num) if $code->run($num);
         }
@@ -241,7 +251,7 @@ package Sidef::Types::Range::RangeNumber {
     sub reduce {
         my ($self, $code) = @_;
 
-        my $iter  = $self->_new_iter();
+        my $iter  = $self->range_iter();
         my $value = $iter->();
 
         while (defined(my $num = $iter->())) {
@@ -254,7 +264,7 @@ package Sidef::Types::Range::RangeNumber {
     sub all {
         my ($self, $code) = @_;
 
-        my $iter = $self->_new_iter();
+        my $iter = $self->range_iter();
         while (defined(my $num = $iter->())) {
             $code->run($num)
               || return Sidef::Types::Bool::Bool::FALSE;
@@ -266,7 +276,7 @@ package Sidef::Types::Range::RangeNumber {
     sub any {
         my ($self, $code) = @_;
 
-        my $iter = $self->_new_iter();
+        my $iter = $self->range_iter();
         while (defined(my $num = $iter->())) {
             $code->run($num)
               && return Sidef::Types::Bool::Bool::TRUE;
@@ -292,7 +302,7 @@ package Sidef::Types::Range::RangeNumber {
         my ($name) = (defined($AUTOLOAD) ? ($AUTOLOAD =~ /^.*[^:]::(.*)$/) : '');
 
         my $array = Sidef::Types::Array::Array->new;
-        my $iter  = $self->_new_iter();
+        my $iter  = $self->range_iter();
         while (defined(my $num = $iter->())) {
             push @$array, $num;
         }
