@@ -248,7 +248,6 @@ package Sidef::Types::String::String {
     }
 
     *eq = \&equals;
-    *is = \&equals;
 
     sub ne {
         my ($self, $arg) = @_;
@@ -681,7 +680,7 @@ package Sidef::Types::String::String {
           }
           : 0;
 
-        if (CORE::not defined $sep) {
+        if (!defined($sep)) {
             return
               Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) }
                                                 split(' ', $$self, $size));
@@ -707,7 +706,7 @@ package Sidef::Types::String::String {
         my ($self, $block) = @_;
 
         if (defined $block) {
-            return $self->to_chars->sort($block)->join;
+            return $self->chars->sort($block)->join;
         }
 
         $self->new(CORE::join('', sort(CORE::split(//, $$self))));
@@ -802,6 +801,9 @@ package Sidef::Types::String::String {
         Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } CORE::split(//, CORE::join('', $$self)));
     }
 
+    *to_a     = \&chars;
+    *to_array = \&chars;
+
     sub each_char {
         my ($self, $code) = @_;
 
@@ -894,7 +896,7 @@ package Sidef::Types::String::String {
         my ($self, $orig, $repl) = @_;
 
         my %map;
-        if (CORE::not defined($repl) and defined($orig)) {    # assume an array of pairs
+        if (!defined($repl) and defined($orig)) {    # assume an array of pairs
             foreach my $pair (@{$orig}) {
                 $map{"$pair->[0]"} = "$pair->[1]";
             }
@@ -1039,7 +1041,7 @@ package Sidef::Types::String::String {
 
         foreach my $i (0 .. $#s) {
             $s_matches[$i] or next;
-            while (not $t_matches[$k]) { ++$k }
+            until ($t_matches[$k]) { ++$k }
             $s[$i] eq $t[$k] or ++$transpositions;
             ++$k;
         }
@@ -1388,11 +1390,13 @@ package Sidef::Types::String::String {
                 splice @chars, $pair->[0] + $i, 0, $pair->[1];
             }
 
+            state $str_dt = bless({}, 'Sidef::DataTypes::String::String');
+
             my $expr = {
                         $parser->{class} => [
                                              {
-                                              self => $self->new,
-                                              call => [{method => 'super_join'}]
+                                              self => $str_dt,
+                                              call => [{method => 'interpolate'}]
                                              }
                                             ]
                        };
@@ -1493,7 +1497,6 @@ package Sidef::Types::String::String {
         *{__PACKAGE__ . '::' . '÷'}  = \&div;
         *{__PACKAGE__ . '::' . '/'}   = \&div;
         *{__PACKAGE__ . '::' . '..'}  = \&array_to;
-        *{__PACKAGE__ . '::' . '...'} = \&to;
         *{__PACKAGE__ . '::' . '..^'} = \&to;
         *{__PACKAGE__ . '::' . '^..'} = \&downto;
         *{__PACKAGE__ . '::' . '^'}   = \&xor;
