@@ -1,12 +1,18 @@
 package Sidef::Variable::NamedParam {
 
-    #use overload q{""} => \&dump;
+    use 5.014;
+    use overload q{""} => \&dump;
 
     sub new {
         my (undef, $name, @args) = @_;
         bless [$name, \@args], __PACKAGE__;
     }
 
+    sub get_value {
+        my ($self) = @_;
+        map { $_->get_value } @{$self->[1]};
+    }
+
     sub dump {
         my ($self) = @_;
         my ($name, $args) = @{$self};
